@@ -43,7 +43,6 @@
 namespace Monero {
 class PendingTransactionImpl;
 class UnsignedTransactionImpl;
-class AddressBookImpl;
 class PendingTransactionInfoImpl;
 class TransactionConstructionInfoImpl;
 struct Wallet2CallbackImpl;
@@ -199,6 +198,13 @@ public:
     bool importOutputsFromStr(const std::string &outputs) override;
     bool scanTransactions(const std::vector<std::string> &txids) override;
 
+    bool setupBackgroundSync(const BackgroundSyncType background_sync_type, const std::string &wallet_password, const std::optional<std::string> &background_cache_password = std::optional<std::string>()) override;
+    BackgroundSyncType getBackgroundSyncType() const override;
+    bool startBackgroundSync() override;
+    bool stopBackgroundSync(const std::string &wallet_password) override;
+    bool isBackgroundSyncing() const override;
+    bool isBackgroundWallet() const override;
+
     virtual std::string printBlockchain() override;
     virtual std::string printTransfers() override;
     virtual std::string printPayments() override;
@@ -222,6 +228,7 @@ public:
     virtual void disposeTransaction(PendingTransaction * t) override;
     virtual uint64_t estimateTransactionFee(const std::vector<std::pair<std::string, uint64_t>> &destinations,
                                             PendingTransaction::Priority priority) const override;
+
     virtual void setListener(WalletListener * l) override;
     virtual uint32_t defaultMixin() const override;
     virtual void setDefaultMixin(uint32_t arg) override;
@@ -284,12 +291,12 @@ private:
     bool daemonSynced() const;
     bool isNewWallet() const;
     void pendingTxPostProcess(PendingTransactionImpl * pending);
+    bool checkBackgroundSync(const std::string &message) const;
 
 private:
     friend class PendingTransactionImpl;
     friend class UnsignedTransactionImpl;
     friend struct Wallet2CallbackImpl;
-    friend class AddressBookImpl;
     friend class PendingTransactionInfoImpl;
     friend class TransactionConstructionInfoImpl;
 
