@@ -1,5 +1,5 @@
 // SPDX-License-Identifier: BSD-3-Clause
-// SPDX-FileCopyrightText: 2020-2024 The Monero Project
+// SPDX-FileCopyrightText: The Monero Project
 
 #include "SettingsDialog.h"
 #include "ui_SettingsDialog.h"
@@ -308,6 +308,21 @@ void Settings::setupDisplayTab() {
     connect(ui->checkBox_lockOnMinimize, &QCheckBox::toggled, [](bool toggled){
         conf()->set(Config::lockOnMinimize, toggled);
     });
+
+    // [Show tray icon]
+    ui->checkBox_showTrayIcon->setChecked(conf()->get(Config::showTrayIcon).toBool());
+    connect(ui->checkBox_showTrayIcon, &QCheckBox::toggled, [this](bool toggled) {
+        conf()->set(Config::showTrayIcon, toggled);
+        ui->checkBox_minimizeToTray->setEnabled(toggled);
+        emit showTrayIcon(toggled);
+    });
+
+    // [Hide window to system tray when minimized]
+    ui->checkBox_minimizeToTray->setEnabled(ui->checkBox_showTrayIcon->isChecked());
+    ui->checkBox_minimizeToTray->setChecked(conf()->get(Config::minimizeToTray).toBool());
+    connect(ui->checkBox_minimizeToTray, &QCheckBox::toggled, [this](bool toggled) {
+        conf()->set(Config::minimizeToTray, toggled);
+    });
 }
 
 void Settings::setupMemoryTab() {
