@@ -1,5 +1,5 @@
 // SPDX-License-Identifier: BSD-3-Clause
-// SPDX-FileCopyrightText: 2020-2024 The Monero Project
+// SPDX-FileCopyrightText: The Monero Project
 
 #include "WindowManager.h"
 
@@ -45,7 +45,7 @@ WindowManager::WindowManager(QObject *parent)
     m_tray = new QSystemTrayIcon(icons()->icon("appicons/64x64.png"));
     m_tray->setToolTip("Feather Wallet");
     this->buildTrayMenu();
-    m_tray->show();
+    m_tray->setVisible(conf()->get(Config::showTrayIcon).toBool());
 
     this->initSkins();
     this->patchMacStylesheet();
@@ -194,6 +194,9 @@ void WindowManager::showSettings(Nodes *nodes, QWidget *parent, bool showProxyTa
             window->onHideUpdateNotifications(hidden);
         }
     });
+    connect(&settings, &Settings::showTrayIcon, [this](bool visible) {
+        m_tray->setVisible(visible);
+    });
     connect(&settings, &Settings::pluginConfigured, [this](const QString &id) {
         emit pluginConfigured(id);
     });
@@ -608,7 +611,7 @@ void WindowManager::onDeviceButtonPressed() {
     m_splashDialog->hide();
 }
 
-void WindowManager::onDeviceError(const QString &errorMessage) {
+void WindowManager::onDeviceError(const QString &errorMessage, quint64 errorCode) {
     // TODO: when does this get called?
     qCritical() << Q_FUNC_INFO << errorMessage;
 }
