@@ -1,5 +1,5 @@
 // SPDX-License-Identifier: BSD-3-Clause
-// SPDX-FileCopyrightText: 2020-2024 The Monero Project
+// SPDX-FileCopyrightText: The Monero Project
 
 #ifndef FEATHER_ACCOUNTSWITCHERDIALOG_H
 #define FEATHER_ACCOUNTSWITCHERDIALOG_H
@@ -22,18 +22,19 @@ public:
 
     void update();
 
+protected:
+    void showEvent(QShowEvent *event) override;
+
 private slots:
     void showContextMenu(const QPoint& point);
     void updateSelection();
 
 private:
-    void switchAccount();
+    void switchAccount(const QModelIndex &index);
     void copyLabel();
     void copyBalance();
     void editLabel();
 
-    AccountRow* currentEntry();
-
     QScopedPointer<Ui::AccountSwitcherDialog> ui;
     Wallet *m_wallet;
     SubaddressAccountModel *m_model;
