@@ -1,9 +1,11 @@
 // SPDX-License-Identifier: BSD-3-Clause
-// SPDX-FileCopyrightText: 2020-2024 The Monero Project
+// SPDX-FileCopyrightText: The Monero Project
 
 #include "KeysDialog.h"
 #include "ui_KeysDialog.h"
 
+#include "utils/Utils.h"
+
 KeysDialog::KeysDialog(Wallet *wallet, QWidget *parent)
     : WindowModalDialog(parent)
     , ui(new Ui::KeysDialog)
@@ -12,7 +14,7 @@ KeysDialog::KeysDialog(Wallet *wallet, QWidget *parent)
 
     QString unavailable = "Unavailable: Key is stored on hardware device";
 
-    ui->label_restoreHeight->setText(QString::number(wallet->getWalletCreationHeight()));
+    ui->label_restoreHeight->setText(Utils::formatRestoreHeight(wallet->getWalletCreationHeight()));
     ui->label_primaryAddress->setText(wallet->address(0, 0));
     ui->label_secretSpendKey->setText(wallet->isHwBacked() ? unavailable : wallet->getSecretSpendKey());
     ui->label_secretViewKey->setText(wallet->getSecretViewKey());
