@@ -1,5 +1,5 @@
 // SPDX-License-Identifier: BSD-3-Clause
-// SPDX-FileCopyrightText: 2020-2024 The Monero Project
+// SPDX-FileCopyrightText: The Monero Project
 
 #ifndef FEATHER_OUTPUTINFODIALOG_H
 #define FEATHER_OUTPUTINFODIALOG_H
@@ -19,12 +19,11 @@ class OutputInfoDialog : public WindowModalDialog
 Q_OBJECT
 
 public:
-    explicit OutputInfoDialog(CoinsInfo *cInfo, QWidget *parent = nullptr);
+    explicit OutputInfoDialog(const CoinsInfo &cInfo, QWidget *parent = nullptr);
     ~OutputInfoDialog() override;
 
 private:
     QScopedPointer<Ui::OutputInfoDialog> ui;
 };
 
-
 #endif //FEATHER_OUTPUTINFODIALOG_H
