@@ -1,5 +1,5 @@
 // SPDX-License-Identifier: BSD-3-Clause
-// SPDX-FileCopyrightText: 2020-2024 The Monero Project
+// SPDX-FileCopyrightText: The Monero Project
 
 #ifndef FEATHER_PAYMENTREQUESTDIALOG_H
 #define FEATHER_PAYMENTREQUESTDIALOG_H
@@ -28,6 +28,9 @@ private slots:
     void copyImage();
     void saveImage();
 
+    void calculateCrypto();
+    void calculateFiat();
+
 private:
     QScopedPointer<Ui::PaymentRequestDialog> ui;
     Wallet *m_wallet;
