@@ -1,5 +1,5 @@
 // SPDX-License-Identifier: BSD-3-Clause
-// SPDX-FileCopyrightText: 2020-2024 The Monero Project
+// SPDX-FileCopyrightText: The Monero Project
 
 #ifndef FEATHER_TXPROOFDIALOG_H
 #define FEATHER_TXPROOFDIALOG_H
@@ -19,7 +19,7 @@ class TxProofDialog : public WindowModalDialog
     Q_OBJECT
 
 public:
-    explicit TxProofDialog(QWidget *parent, Wallet *wallet, TransactionRow *txid);
+    explicit TxProofDialog(QWidget *parent, Wallet *wallet, const TransactionRow &txid);
     ~TxProofDialog() override;
     void setTxId(const QString &txid);
     void getTxKey();
