@@ -1,17 +1,13 @@
 // SPDX-License-Identifier: BSD-3-Clause
-// SPDX-FileCopyrightText: 2020-2024 The Monero Project
+// SPDX-FileCopyrightText: The Monero Project
 
-#ifndef ADDRESSBOOK_H
-#define ADDRESSBOOK_H
+#ifndef FEATHER_ADDRESSBOOK_H
+#define FEATHER_ADDRESSBOOK_H
 
-#include <QMap>
 #include <QObject>
-#include <QReadWriteLock>
 #include <QList>
-#include <QDateTime>
 
 #include "rows/ContactRow.h"
-#include "Wallet.h"
 
 namespace Monero {
 struct AddressBook;
@@ -21,6 +17,7 @@ namespace tools{
     class wallet2;
 }
 
+class Wallet;
 class AddressBook : public QObject
 {
     Q_OBJECT
@@ -34,33 +31,32 @@ public:
     };
     Q_ENUM(ErrorCode);
 
-    bool getRow(int index, std::function<void (ContactRow &)> callback) const;
-    bool addRow(const QString &address, const QString &description);
-    bool deleteRow(int rowId);
-    bool setDescription(int index, const QString &label);
+    void refresh();
     qsizetype count() const;
+
+    const ContactRow& getRow(qsizetype index);
+    const QList<ContactRow>& getRows();
+
+    bool addRow(const QString &address, const QString &description);
+    bool setDescription(qsizetype index, const QString &description);
+    bool deleteRow(qsizetype index);
+
     QString errorString() const;
     ErrorCode errorCode() const;
 
-    void refresh();
-    void clearRows();
-
-
 signals:
     void refreshStarted() const;
     void refreshFinished() const;
-    void descriptionChanged() const;
 
 private:
-    explicit AddressBook(Wallet *wallet, tools::wallet2 *wallet2, QObject *parent);
+    explicit AddressBook(tools::wallet2 *wallet2, QObject *parent);
     friend class Wallet;
 
-    Wallet *m_wallet;
     tools::wallet2 *m_wallet2;
-    QList<ContactRow*> m_rows;
+    QList<ContactRow> m_rows;
 
     QString m_errorString;
     ErrorCode m_errorCode;
 };
 
-#endif // ADDRESSBOOK_H
+#endif // FEATHER_ADDRESSBOOK_H
