@@ -1,13 +1,13 @@
 // SPDX-License-Identifier: BSD-3-Clause
-// SPDX-FileCopyrightText: 2020-2024 The Monero Project
+// SPDX-FileCopyrightText: The Monero Project
 
-#ifndef PENDINGTRANSACTION_H
-#define PENDINGTRANSACTION_H
+#ifndef FEATHER_PENDINGTRANSACTION_H
+#define FEATHER_PENDINGTRANSACTION_H
 
 #include <QObject>
 #include <QList>
 
-#include "PendingTransactionInfo.h"
+#include "rows/PendingTransactionInfo.h"
 
 namespace Monero {
     class PendingTransaction;
@@ -42,7 +42,7 @@ public:
     quint64 weight(int index) const;
     void refresh();
 
-    PendingTransactionInfo * transaction(int index) const;
+    const PendingTransactionInfo& transaction(int index) const;
 
 private:
     explicit PendingTransaction(Monero::PendingTransaction * pt, QObject *parent = nullptr);
@@ -50,7 +50,7 @@ private:
 private:
     friend class Wallet;
     Monero::PendingTransaction * m_pimpl;
-    mutable QList<PendingTransactionInfo*> m_pending_tx_info;
+    mutable QList<PendingTransactionInfo> m_pending_tx_info;
 };
 
-#endif // PENDINGTRANSACTION_H
+#endif // FEATHER_PENDINGTRANSACTION_H
