@@ -1,17 +1,12 @@
 // SPDX-License-Identifier: BSD-3-Clause
-// SPDX-FileCopyrightText: 2020-2024 The Monero Project
+// SPDX-FileCopyrightText: The Monero Project
 
-#ifndef SUBADDRESSACCOUNT_H
-#define SUBADDRESSACCOUNT_H
-
-#include <functional>
+#ifndef FEATHER_SUBADDRESSACCOUNT_H
+#define FEATHER_SUBADDRESSACCOUNT_H
 
 #include <QObject>
-#include <QReadWriteLock>
 #include <QList>
-#include <QDateTime>
 
-#include "Wallet.h"
 #include "rows/AccountRow.h"
 
 namespace tools {
@@ -23,30 +18,25 @@ class SubaddressAccount : public QObject
     Q_OBJECT
 
 public:
-    void getAll() const;
-    bool getRow(int index, std::function<void (AccountRow &row)> callback) const;
-    void addRow(const QString &label);
-
-    void setLabel(quint32 accountIndex, const QString &label);
-
     void refresh();
-
     qsizetype count() const;
-    void clearRows();
 
-    AccountRow* row(int index) const;
+    const AccountRow& row(int index) const;
+    const QList<AccountRow>& getRows();
+
+    void addRow(const QString &label);
+    void setLabel(quint32 accountIndex, const QString &label);
 
 signals:
     void refreshStarted() const;
     void refreshFinished() const;
 
 private:
-    explicit SubaddressAccount(Wallet *wallet, tools::wallet2 *wallet2, QObject *parent);
+    explicit SubaddressAccount(tools::wallet2 *wallet2, QObject *parent);
     friend class Wallet;
 
-    Wallet *m_wallet;
     tools::wallet2 *m_wallet2;
-    QList<AccountRow*> m_rows;
+    QList<AccountRow> m_rows;
 };
 
-#endif // SUBADDRESSACCOUNT_H
+#endif // FEATHER_SUBADDRESSACCOUNT_H
