@@ -1,18 +1,12 @@
 // SPDX-License-Identifier: BSD-3-Clause
-// SPDX-FileCopyrightText: 2020-2024 The Monero Project
+// SPDX-FileCopyrightText: The Monero Project
 
-#ifndef TRANSACTIONHISTORY_H
-#define TRANSACTIONHISTORY_H
+#ifndef FEATHER_TRANSACTIONHISTORY_H
+#define FEATHER_TRANSACTIONHISTORY_H
 
-#include <functional>
-
-#include <QObject>
-#include <QList>
 #include <QReadWriteLock>
-#include <QDateTime>
 
 #include "rows/TransactionRow.h"
-#include "Wallet.h"
 
 namespace tools {
     class wallet2;
@@ -23,25 +17,21 @@ struct TransactionHistory;
 }
 
 class TransactionInfo;
-
+class Wallet;
 class TransactionHistory : public QObject
 {
     Q_OBJECT
 
 public:
-    bool transaction(int index, std::function<void (TransactionRow &)> callback);
-    TransactionRow * transaction(const QString &id);
-    TransactionRow* transaction(int index);
     void refresh();
-    void setTxNote(const QString &txid, const QString &note);
     quint64 count() const;
-    QDateTime firstDateTime() const;
-    QDateTime lastDateTime() const;
-    quint64 minutesToUnlock() const;
-    bool locked() const;
-    void clearRows();
 
-    bool writeCSV(const QString &path);
+    const TransactionRow& transaction(int index);
+    const QList<TransactionRow>& getRows();
+
+    void setTxNote(const QString &txid, const QString &note);
+    bool locked() const;
+
     QString importLabelsFromCSV(const QString &fileName);
 
 signals:
@@ -60,7 +50,7 @@ private:
 
     Wallet *m_wallet;
     tools::wallet2 *m_wallet2;
-    QList<TransactionRow*> m_rows;
+    QList<TransactionRow> m_rows;
 
     mutable QDateTime   m_firstDateTime;
     mutable QDateTime   m_lastDateTime;
@@ -71,4 +61,4 @@ private:
     quint32 lastAccountIndex = 0;
 };
 
-#endif // TRANSACTIONHISTORY_H
+#endif // FEATHER_TRANSACTIONHISTORY_H
