@@ -1,8 +1,8 @@
 // SPDX-License-Identifier: BSD-3-Clause
-// SPDX-FileCopyrightText: 2020-2024 The Monero Project
+// SPDX-FileCopyrightText: The Monero Project
 
-#ifndef UNSIGNEDTRANSACTION_H
-#define UNSIGNEDTRANSACTION_H
+#ifndef FEATHER_UNSIGNEDTRANSACTION_H
+#define FEATHER_UNSIGNEDTRANSACTION_H
 
 #include <QObject>
 
@@ -41,7 +41,7 @@ public:
     void setFilename(const QString &fileName);
     void refresh();
 
-    ConstructionInfo * constructionInfo(int index) const;
+    const ConstructionInfo& constructionInfo(int index) const;
 
 private:
     explicit UnsignedTransaction(Monero::UnsignedTransaction * pt, Monero::Wallet *walletImpl, QObject *parent = nullptr);
@@ -51,7 +51,7 @@ private:
     Monero::UnsignedTransaction * m_pimpl;
     QString m_fileName;
     Monero::Wallet * m_walletImpl;
-    mutable QList<ConstructionInfo*> m_construction_info;
+    mutable QList<ConstructionInfo> m_construction_info;
 };
 
-#endif // UNSIGNEDTRANSACTION_H
+#endif // FEATHER_UNSIGNEDTRANSACTION_H
