@@ -1,5 +1,5 @@
 // SPDX-License-Identifier: BSD-3-Clause
-// SPDX-FileCopyrightText: 2020-2024 The Monero Project
+// SPDX-FileCopyrightText: The Monero Project
 
 #include "Wallet.h"
 
@@ -38,14 +38,14 @@ Wallet::Wallet(Monero::Wallet *wallet, QObject *parent)
         , m_wallet2(wallet->getWallet())
         , m_history(new TransactionHistory(this, wallet->getWallet(), this))
         , m_historyModel(nullptr)
-        , m_addressBook(new AddressBook(this, wallet->getWallet(), this))
+        , m_addressBook(new AddressBook(wallet->getWallet(), this))
         , m_addressBookModel(nullptr)
         , m_daemonBlockChainHeight(0)
         , m_daemonBlockChainTargetHeight(0)
         , m_connectionStatus(Wallet::ConnectionStatus_Disconnected)
         , m_currentSubaddressAccount(0)
         , m_subaddress(new Subaddress(this, wallet->getWallet(), this))
-        , m_subaddressAccount(new SubaddressAccount(this, wallet->getWallet(), this))
+        , m_subaddressAccount(new SubaddressAccount(wallet->getWallet(), this))
         , m_refreshNow(false)
         , m_refreshEnabled(false)
         , m_scheduler(this)
@@ -261,6 +261,16 @@ QString Wallet::getAddressSafe(quint32 accountIndex, quint32 addressIndex, bool
         return {};
     }
 
+    if (!rct::isInMainSubgroup(rct::pk2rct(info.address.m_spend_public_key))) {
+        reason = "Spend public key is not is main subgroup";
+        return {};
+    }
+
+    if (!rct::isInMainSubgroup(rct::pk2rct(info.address.m_view_public_key))) {
+        reason = "View public key is not in main subgroup";
+        return {};
+    }
+
     ok = true;
     return QString::fromStdString(address_str);
 }
@@ -285,12 +295,12 @@ void Wallet::switchSubaddressAccount(quint32 accountIndex) {
         {
             qWarning() << "failed to set " << ATTRIBUTE_SUBADDRESS_ACCOUNT << " cache attribute";
         }
-        m_subaddress->refresh(m_currentSubaddressAccount);
+        m_subaddress->refresh();
         m_history->refresh();
         m_coins->refresh();
-        this->subaddressModel()->setCurrentSubaddressAccount(m_currentSubaddressAccount);
         this->coinsModel()->setCurrentSubaddressAccount(m_currentSubaddressAccount);
         this->updateBalance();
+        this->setSelectedInputs({});
         emit currentSubaddressAccountChanged();
     }
 }
@@ -307,18 +317,10 @@ quint32 Wallet::numSubaddresses(quint32 accountIndex) const {
     return m_wallet2->get_num_subaddresses(accountIndex);
 }
 
-void Wallet::addSubaddress(const QString& label) {
-    m_wallet2->add_subaddress(currentSubaddressAccount(), label.toStdString());
-}
-
 QString Wallet::getSubaddressLabel(quint32 accountIndex, quint32 addressIndex) const {
     return QString::fromStdString(m_walletImpl->getSubaddressLabel(accountIndex, addressIndex));
 }
 
-void Wallet::setSubaddressLabel(quint32 accountIndex, quint32 addressIndex, const QString &label) {
-    m_walletImpl->setSubaddressLabel(accountIndex, addressIndex, label.toStdString());
-}
-
 void Wallet::deviceShowAddressAsync(quint32 accountIndex, quint32 addressIndex, const QString &paymentId) {
     m_scheduler.run([this, accountIndex, addressIndex, paymentId] {
         m_walletImpl->deviceShowAddress(accountIndex, addressIndex, paymentId.toStdString());
@@ -331,6 +333,15 @@ QString Wallet::getSubaddressLookahead() const {
     return QString("%1:%2").arg(QString::number(lookahead.first), QString::number(lookahead.second));
 }
 
+bool Wallet::isAddressTorsionFree(const QString& address) {
+    cryptonote::address_parse_info info;
+    bool r = cryptonote::get_account_address_from_str(info, m_wallet2->nettype(), address.toStdString());
+    if (!r) {
+        return false;
+    }
+    return rct::isInMainSubgroup(rct::pk2rct(info.address.m_spend_public_key)) && rct::isInMainSubgroup(rct::pk2rct(info.address.m_view_public_key));
+}
+
 // #################### Seed ####################
 
 QString Wallet::getSeed(const QString &seedOffset) const {
@@ -420,6 +431,7 @@ void Wallet::initAsync(const QString &daemonAddress, bool trustedDaemon, quint64
 // #################### Synchronization (Refresh) ####################
 
 void Wallet::startRefresh() {
+    m_refreshEnabled = true;
     m_refreshEnabled = true;
     m_refreshNow = true;
 }
@@ -545,14 +557,18 @@ void Wallet::onNewBlock(uint64_t walletHeight) {
     this->syncStatusUpdated(walletHeight, daemonHeight);
 
     if (this->isSynchronized()) {
-        this->refreshModels();
+        m_history->refresh();
+        m_coins->refresh();
+        this->subaddress()->updateUsed(this->currentSubaddressAccount());
     }
 }
 
 void Wallet::onUpdated() {
     this->updateBalance();
     if (this->isSynchronized()) {
-        this->refreshModels();
+        m_history->refresh();
+        m_coins->refresh();
+        this->subaddress()->updateUsed(this->currentSubaddressAccount());
     }
 }
 
@@ -575,7 +591,7 @@ void Wallet::onRefreshed(bool success, const QString &message) {
 void Wallet::refreshModels() {
     m_history->refresh();
     m_coins->refresh();
-    this->subaddress()->refresh(this->currentSubaddressAccount());
+    m_subaddress->refresh();
 }
 
 // #################### Hardware wallet ####################
@@ -984,6 +1000,8 @@ void Wallet::onTransactionCommitted(bool success, PendingTransaction *tx, const
 
     this->history()->refresh();
     this->coins()->refresh();
+    this->subaddress()->refresh();
+
     this->updateBalance();
 
     if (!success) {
