@@ -1,8 +1,8 @@
 // SPDX-License-Identifier: BSD-3-Clause
-// SPDX-FileCopyrightText: 2020-2024 The Monero Project
+// SPDX-FileCopyrightText: The Monero Project
 
-#ifndef WALLET_H
-#define WALLET_H
+#ifndef FEATHER_WALLET_H
+#define FEATHER_WALLET_H
 
 #include <QObject>
 #include <QMutex>
@@ -170,12 +170,12 @@ public:
     void addSubaddressAccount(const QString& label);
     quint32 numSubaddressAccounts() const;
     quint32 numSubaddresses(quint32 accountIndex) const;
-    void addSubaddress(const QString& label);
     QString getSubaddressLabel(quint32 accountIndex, quint32 addressIndex) const;
-    void setSubaddressLabel(quint32 accountIndex, quint32 addressIndex, const QString &label);
     void deviceShowAddressAsync(quint32 accountIndex, quint32 addressIndex, const QString &paymentId);
     QString getSubaddressLookahead() const;
 
+    bool isAddressTorsionFree(const QString& address);
+
     // ##### Seed #####
 
     //! returns mnemonic seed
@@ -441,7 +441,7 @@ signals:
     void walletCreationHeightChanged();
     void deviceButtonRequest(quint64 buttonCode);
     void deviceButtonPressed();
-    void deviceError(const QString &message);
+    void deviceError(const QString &message, quint64 errorCode);
     void walletPassphraseNeeded(bool onDevice);
     void beginCommitTransaction();
     void transactionCommitted(bool status, PendingTransaction *t, const QStringList& txid, const QMap<QString, QString> &txHexMap);
@@ -533,4 +533,4 @@ private:
     std::set<std::string> m_selectedInputs;
 };
 
-#endif // WALLET_H
+#endif // FEATHER_WALLET_H
