@@ -1,5 +1,5 @@
 // SPDX-License-Identifier: BSD-3-Clause
-// SPDX-FileCopyrightText: 2020-2024 The Monero Project
+// SPDX-FileCopyrightText: The Monero Project
 
 #ifndef FEATHER_COINSMODEL_H
 #define FEATHER_COINSMODEL_H
@@ -31,8 +31,6 @@ public:
 
     explicit CoinsModel(QObject *parent, Coins *coins);
 
-    Coins * coins() const;
-
     int rowCount(const QModelIndex &parent = QModelIndex()) const override;
     int columnCount(const QModelIndex &parent = QModelIndex()) const override;
     QVariant data(const QModelIndex &index, int role = Qt::DisplayRole) const override;
@@ -40,7 +38,7 @@ public:
     Qt::ItemFlags flags(const QModelIndex &index) const override;
     bool setData(const QModelIndex &index, const QVariant &value, int role) override;
 
-    CoinsInfo* entryFromIndex(const QModelIndex &index) const;
+    const CoinsInfo& entryFromIndex(const QModelIndex &index) const;
 
     void setCurrentSubaddressAccount(quint32 accountIndex);
     void setSelected(const QStringList &selected);
@@ -48,10 +46,6 @@ public:
 signals:
     void descriptionChanged();
 
-public slots:
-    void startReset();
-    void endReset();
-
 private:
     QVariant parseTransactionInfo(const CoinsInfo &cInfo, int column, int role) const;
 
