@@ -1,5 +1,5 @@
 // SPDX-License-Identifier: BSD-3-Clause
-// SPDX-FileCopyrightText: 2020-2024 The Monero Project
+// SPDX-FileCopyrightText: The Monero Project
 
 #ifndef FEATHER_HISTORYVIEW_H
 #define FEATHER_HISTORYVIEW_H
@@ -18,11 +18,14 @@ class HistoryView : public QTreeView
 public:
     explicit HistoryView(QWidget* parent = nullptr);
     void setHistoryModel(TransactionHistoryProxyModel *model);
-    TransactionRow* currentEntry();
 
     void setSearchMode(bool mode);
     QByteArray viewState() const;
     bool setViewState(const QByteArray& state);
+    QModelIndex getCurrentIndex();
+    TransactionHistoryModel* sourceModel();
+
+    QMenu* getMenu();
 
 private slots:
     void showHeaderMenu(const QPoint& position);
@@ -36,8 +39,6 @@ protected:
     void keyPressEvent(QKeyEvent *event);
 
 private:
-    TransactionHistoryModel* sourceModel();
-
     TransactionHistoryProxyModel* m_model;
     bool m_inSearchMode = false;
     bool m_columnsNeedRelayout = true;
