@@ -1,5 +1,5 @@
 // SPDX-License-Identifier: BSD-3-Clause
-// SPDX-FileCopyrightText: 2020-2024 The Monero Project
+// SPDX-FileCopyrightText: The Monero Project
 
 #ifndef SUBADDRESSMODEL_H
 #define SUBADDRESSMODEL_H
@@ -34,9 +34,10 @@ public:
 
     bool setData(const QModelIndex &index, const QVariant &value, int role) override;
 
-    SubaddressRow* entryFromIndex(const QModelIndex &index) const;
+    const SubaddressRow& entryFromIndex(const QModelIndex &index) const;
 
-    void setCurrentSubaddressAccount(quint32 accountIndex);
+    void rowUpdated(qsizetype index);
+    void beginRowAdded(qsizetype index);
 
 private:
     Subaddress *m_subaddress;
