@@ -1,5 +1,5 @@
 // SPDX-License-Identifier: BSD-3-Clause
-// SPDX-FileCopyrightText: 2020-2024 The Monero Project
+// SPDX-FileCopyrightText: The Monero Project
 
 #include "Utils.h"
 
@@ -20,6 +20,7 @@
 
 #include "constants.h"
 #include "networktype.h"
+#include "utils/AppData.h"
 #include "utils/ColorScheme.h"
 #include "utils/config.h"
 #include "utils/os/tails.h"
@@ -169,6 +170,7 @@ QString defaultWalletDir() {
     }
 
     if (TailsOS::detect()) {
+#ifdef SELF_CONTAINED
         QString path = []{
             // Starting in 1.1.0 the wallet and config directory were moved from ./.feather to ./feather_data
             // A user might accidentally delete the folder containing the file hidden folder after moving the AppImage
@@ -192,6 +194,11 @@ QString defaultWalletDir() {
         }();
 
         return path;
+#else
+        if (TailsOS::detectDataPersistence()) {
+            return QDir::homePath() + "/Persistent/feather_data/wallets";
+        }
+#endif
     }
 
 #if defined(Q_OS_LINUX) or defined(Q_OS_MAC)
@@ -331,12 +338,23 @@ bool xdgDesktopEntryRegister() {
 #endif
 
     QPixmap appIcon(":assets/images/appicons/64x64.png");
-    QString pathIcon = QString("%1/.local/share/icons/feather.png").arg(QDir::homePath());
-    if (!fileExists(pathIcon)) {
-        pixmapWrite(pathIcon, appIcon);
+    QString iconPathSuffix = "%1/.local/share/icons/feather.png";
+    QString iconPath = iconPathSuffix.arg(QDir::homePath());
+    if (!fileExists(iconPath)) {
+        pixmapWrite(iconPath, appIcon);
     }
     xdgDesktopEntryWrite(QStandardPaths::writableLocation(QStandardPaths::ApplicationsLocation) + "/feather-wallet.desktop");
 
+    // Also write to dotfiles persistence
+    if (TailsOS::detect() && TailsOS::detectDotPersistence()) {
+        QString basePath = "/live/persistence/TailsData_unlocked/dotfiles";
+        iconPath = iconPathSuffix.arg(basePath);
+        if (!fileExists(iconPath)) {
+            pixmapWrite(iconPath, appIcon);
+        }
+        xdgDesktopEntryWrite(basePath + "/.local/share/applications/feather-wallet.desktop");
+    }
+
     xdgRefreshApplications();
     return true;
 }
@@ -693,4 +711,17 @@ QString formatSyncStatus(quint64 height, quint64 target, bool daemonSync) {
 
     return "Synchronized";
 }
+
+QString formatRestoreHeight(quint64 height) {
+    const QDateTime restoreDate = appData()->restoreHeights[constants::networkType]->heightToDate(height);
+    return QString("%1  (%2)").arg(QString::number(height), restoreDate.toString("yyyy-MM-dd"));
+}
+
+QString getVersion() {
+    QString version = QString("%1").arg(FEATHER_VERSION);
+#ifdef OFFICIAL_BUILD
+    version += " (release)";
+#endif
+    return version;
+}
 }
